@@ -1,5 +1,5 @@
 /* mpdscribble (MPD Client)
- * Copyright (C) 2008-2009 The Music Player Daemon Project
+ * Copyright (C) 2008-2010 The Music Player Daemon Project
  * Copyright (C) 2005-2008 Kuno Woudt <kuno@frob.nl>
  * Project homepage: http://musicpd.org
  
@@ -23,6 +23,7 @@
 #include "journal.h"
 #include "http_client.h"
 #include "config.h"
+#include "log.h" /* for log_date() */
 #include "compat.h"
 
 #include <glib.h>
@@ -84,6 +85,8 @@ typedef enum {
 struct scrobbler {
 	const struct scrobbler_config *config;
 
+	FILE *file;
+
 	enum scrobbler_state state;
 
 	unsigned interval;
@@ -121,6 +124,7 @@ scrobbler_new(const struct scrobbler_config *config)
 	struct scrobbler *scrobbler = g_new(struct scrobbler, 1);
 
 	scrobbler->config = config;
+	scrobbler->file = NULL;
 	scrobbler->state = SCROBBLER_STATE_NOTHING;
 	scrobbler->interval = 1;
 	scrobbler->handshake_source_id = 0;
@@ -163,6 +167,10 @@ scrobbler_free(struct scrobbler *scrobbler)
 	g_free(scrobbler->session);
 	g_free(scrobbler->nowplay_url);
 	g_free(scrobbler->submit_url);
+
+	if (scrobbler->file != NULL)
+		fclose(scrobbler->file);
+
 	g_free(scrobbler);
 }
 
@@ -314,6 +322,7 @@ scrobbler_handshake_callback(size_t length, const char *response, void *data)
 	bool ret;
 
 	assert(scrobbler != NULL);
+	assert(scrobbler->config->file == NULL);
 	assert(scrobbler->state == SCROBBLER_STATE_HANDSHAKE);
 
 	scrobbler->state = SCROBBLER_STATE_NOTHING;
@@ -385,6 +394,7 @@ scrobbler_submit_callback(size_t length, const char *response, void *data)
 	struct scrobbler *scrobbler = data;
 	char *newline;
 
+	assert(scrobbler->config->file == NULL);
 	assert(scrobbler->state == SCROBBLER_STATE_SUBMITTING);
 	scrobbler->state = SCROBBLER_STATE_READY;
 
@@ -496,6 +506,8 @@ static char *as_md5(const char *password, const char *timestamp)
 static void
 scrobbler_handshake(struct scrobbler *scrobbler)
 {
+	assert(scrobbler->config->file == NULL);
+
 	GString *url;
 	char *timestr, *md5;
 
@@ -530,6 +542,7 @@ scrobbler_handshake_timer(gpointer data)
 {
 	struct scrobbler *scrobbler = data;
 
+	assert(scrobbler->config->file == NULL);
 	assert(scrobbler->state == SCROBBLER_STATE_NOTHING);
 
 	scrobbler->handshake_source_id = 0;
@@ -541,6 +554,7 @@ scrobbler_handshake_timer(gpointer data)
 static void
 scrobbler_schedule_handshake(struct scrobbler *scrobbler)
 {
+	assert(scrobbler->config->file == NULL);
 	assert(scrobbler->state == SCROBBLER_STATE_NOTHING);
 	assert(scrobbler->handshake_source_id == 0);
 
@@ -552,11 +566,13 @@ scrobbler_schedule_handshake(struct scrobbler *scrobbler)
 static void
 scrobbler_send_now_playing(struct scrobbler *scrobbler, const char *artist,
 			   const char *track, const char *album,
+			   const char *number,
 			   const char *mbid, const int length)
 {
 	GString *post_data;
 	char len[16];
 
+	assert(scrobbler->config->file == NULL);
 	assert(scrobbler->state == SCROBBLER_STATE_READY);
 	assert(scrobbler->submit_source_id == 0);
 
@@ -570,7 +586,7 @@ scrobbler_send_now_playing(struct scrobbler *scrobbler, const char *artist,
 	add_var(post_data, "t", track);
 	add_var(post_data, "b", album);
 	add_var(post_data, "l", len);
-	add_var(post_data, "n", "");
+	add_var(post_data, "n", number);
 	add_var(post_data, "m", mbid);
 
 	g_message("[%s] sending 'now playing' notification",
@@ -589,6 +605,10 @@ scrobbler_schedule_now_playing_callback(gpointer data, gpointer user_data)
 	struct scrobbler *scrobbler = data;
 	const struct record *song = user_data;
 
+	if (scrobbler->file != NULL)
+		/* there's no "now playing" support for files */
+		return;
+
 	record_deinit(&scrobbler->now_playing);
 	record_copy(&scrobbler->now_playing, song);
 
@@ -599,13 +619,15 @@ scrobbler_schedule_now_playing_callback(gpointer data, gpointer user_data)
 
 void
 as_now_playing(const char *artist, const char *track,
-	       const char *album, const char *mbid, const int length)
+	       const char *album, const char *number,
+	       const char *mbid, const int length)
 {
 	struct record record;
 
 	record.artist = g_strdup(artist);
 	record.track = g_strdup(track);
 	record.album = g_strdup(album);
+	record.number = g_strdup(number);
 	record.mbid = g_strdup(mbid);
 	record.time = NULL;
 	record.length = length;
@@ -623,6 +645,7 @@ scrobbler_submit(struct scrobbler *scrobbler)
 	unsigned count = 0;
 	GString *post_data;
 
+	assert(scrobbler->config->file == NULL);
 	assert(scrobbler->state == SCROBBLER_STATE_READY);
 	assert(scrobbler->submit_source_id == 0);
 
@@ -634,6 +657,7 @@ scrobbler_submit(struct scrobbler *scrobbler)
 						   scrobbler->now_playing.artist,
 						   scrobbler->now_playing.track,
 						   scrobbler->now_playing.album,
+						   scrobbler->now_playing.number,
 						   scrobbler->now_playing.mbid,
 						   scrobbler->now_playing.length);
 
@@ -661,7 +685,7 @@ scrobbler_submit(struct scrobbler *scrobbler)
 		add_var_i(post_data, "o", count, song->source);
 		add_var_i(post_data, "r", count, "");
 		add_var_i(post_data, "b", count, song->album);
-		add_var_i(post_data, "n", count, "");
+		add_var_i(post_data, "n", count, song->number);
 		add_var_i(post_data, "m", count, song->mbid);
 
 		count++;
@@ -687,6 +711,13 @@ scrobbler_push_callback(gpointer data, gpointer user_data)
 	struct scrobbler *scrobbler = data;
 	const struct record *record = user_data;
 
+	if (scrobbler->file != NULL) {
+		fprintf(scrobbler->file, "%s %s - %s\n",
+			log_date(), record->artist, record->track);
+		fflush(scrobbler->file);
+		return;
+	}
+
 	g_queue_push_tail(scrobbler->queue, record_dup(record));
 
 	if (scrobbler->state == SCROBBLER_STATE_READY &&
@@ -696,7 +727,8 @@ scrobbler_push_callback(gpointer data, gpointer user_data)
 
 void
 as_songchange(const char *file, const char *artist, const char *track,
-	      const char *album, const char *mbid, const int length,
+	      const char *album, const char *number,
+	      const char *mbid, const int length,
 	      const char *time2)
 {
 	struct record record;
@@ -723,6 +755,7 @@ as_songchange(const char *file, const char *artist, const char *track,
 	record.artist = g_strdup(artist);
 	record.track = g_strdup(track);
 	record.album = g_strdup(album);
+	record.number = g_strdup(number);
 	record.mbid = g_strdup(mbid);
 	record.length = length;
 	record.time = time2 ? g_strdup(time2) : as_timestamp();
@@ -733,6 +766,8 @@ as_songchange(const char *file, const char *artist, const char *track,
 		  record.track, record.length);
 
 	g_slist_foreach(scrobblers, scrobbler_push_callback, &record);
+
+	record_deinit(&record);
 }
 
 static void
@@ -753,7 +788,13 @@ scrobbler_new_callback(gpointer data, G_GNUC_UNUSED gpointer user_data)
 	}
 
 	scrobblers = g_slist_prepend(scrobblers, scrobbler);
-	scrobbler_schedule_handshake(scrobbler);
+	if (config->file != NULL) {
+		scrobbler->file = fopen(config->file, "a");
+		if (scrobbler->file == NULL)
+			g_error("Failed to open file '%s' of scrobbler '%s': %s\n",
+				config->file, config->name, g_strerror(errno));
+	} else
+		scrobbler_schedule_handshake(scrobbler);
 }
 
 void as_init(GSList *scrobbler_configs)
@@ -793,7 +834,7 @@ scrobbler_save_callback(gpointer data, G_GNUC_UNUSED gpointer user_data)
 {
 	struct scrobbler *scrobbler = data;
 
-	if (scrobbler->config->journal == NULL)
+	if (scrobbler->file != NULL || scrobbler->config->journal == NULL)
 		return;
 
 	if (journal_write(scrobbler->config->journal, scrobbler->queue)) {
